;*********************************************************************************************

;	V A N T R I S

;	by 

;	Norbert Kehrer


;	Vantris is a Tetris-style game for the Vanguard arcade machine.
;	The Vanguard game was made 1981 by the SNK Corporation.

;	I thought that a classic game runs best on a classic machine.

;	It really is great fun to make new software for such old arcade machines.
;	Thanks to the MAME team for their great emulation engines !!

;	This program was tested with the M.A.M.E. Emulator.
;	It wonder if the program works on a real machine. Would someone, who owns
;	such a beast, burn the software into a ROM and test it on the real hardware
;	for which it was written.
;	Let me know of your efforts.

;*********************************************************************************************
;	Norbert Kehrer, programmed in October and November 1998 in Austria/Europe
;	e-mail:    norbert.kehrer@utanet.at
;	homepage:  http://web.utanet.at/nkehrer/
;*********************************************************************************************


;	Bug fixes:

;	- 15th of January, 2000:
;	     Block could land on the upper edge of the walls when moved to the outer left
;	     or right position.
;	     Thanks to Brian Deuel (http://coinop.vintagegaming.com) for reporting 


;*********************************************************************************************
; 	Main program, interrupt routines, scoring routines, and sound



;	Vanguard memory map (preliminary) - taken from the MAME source code

;	0000-03ff RAM
;	0400-07ff Video RAM 1
;	0800-0bff Video RAM 2
;	0c00-0fff Color RAM (3 bits for video RAM 1 and 3 bits for video RAM 2)
;	1000-1fff Character generator RAM
;	4000-bfff ROM

;	read:
;	3104      IN0
;	3105      IN1
;	3106      DSW ??
;	3107      IN2

;	write
;	3100      Sound Port 0
;	3101      Sound Port 1
;	3103      bit 7 = flip screen
;	          bit 4-6 = coin counter?
;	3200      y scroll register
;	3300      x scroll register




;**************************************************
;	**** Constants
;**************************************************

; Memory Addresses of the Machine
VIDEO_RAM_1	equ $0400
VIDEO_RAM_2	equ $0800
COLOR_RAM	equ $0c00
SOUND_0		equ $3100
SOUND_1		equ $3101
IN_PORT_0	equ $3104
IN_PORT_2	equ $3107
SCROLL_Y	equ $3200
SCROLL_X	equ $3300

; Controls
VRAM_1		equ $01		; foreground
VRAM_2		equ $00		; background
DRAW		equ $ff		; draw block
DELETE		equ $00		; delete block
MOVING		equ $80		; moving block
FIXED		equ $00		; fixed block

; Coordinates and Sizes of Screen Objects
PLAYF_WIDTH	equ 10		; playfield width
BLOCK_START_X	equ 4		; x-coo where new blocks are positioned
CREDITS_X	equ 22		; x-coo where credits are shown on the screen
CREDITS_Y	equ 16		; y-coo where credits are shown on the screen
SCORE_X		equ 20		; x-coo where score is shown on the screen
SCORE_Y		equ 6		; y-coo where score is shown on the screen
HIGH_SCORE_X	equ 20		; x-coo where high score is shown on the screen
HIGH_SCORE_Y	equ 11		; y-coo where high score is shown on the screen


; Speed control
INITIAL_SPEED	equ 30		; speed at beginning of game
MAXIMUM_SPEED	equ 3		; maximum speed
FAST_SPEED	equ 3		; speed, when joystick is down (fast mode)
SPEED_INCREASE	equ 5		; nr. of full rows that initiates speed increase
AUTO_REP_SPEED	equ 10		; speed for auto repeat for left and right movement

; Movements
MOVE_NOT	equ $00
MOVE_LEFT	equ $01
MOVE_RIGHT	equ $02
MOVE_DOWN	equ $03
MOVE_TURN	equ $04

; Boolean constants
TRUE		equ $00
FALSE		equ $01


; Sounds
SND_QUIET	equ $08		; silent night
LEN_QUIET	equ 1
SND_LANDING	equ $31		; block lands
LEN_LANDING	equ 50
SND_FULLROW	equ $32		; row is filled
LEN_FULLROW	equ 50	
SND_COIN	equ $33		; coin was inserted
LEN_COIN	equ 50
SND_NEW_GAME	equ $34		; new game begins
LEN_NEW_GAME	equ 50
SND_GAME_OVER	equ $35		; game is over
LEN_GAME_OVER	equ 50


;**************************************************
;	**** Variables
;**************************************************

print_addr	equ $10		; Address of string to print (2 bytes)
block_desc	equ $12		; Address of current block description

h_word_A	equ $20		; intermediary (2 bytes)
h_word_B	equ $22		; intermediary (2 bytes)

h_byte_A	equ $0300	; intermediary (1 byte)
h_byte_B	equ $0301	; intermediary (1 byte)


curr_color	equ $0306	; current print color (1 byte)
ps_x		equ $0307	; print location x (1 byte)
ps_y		equ $0308	; print location y (1 byte)
ps_cnt		equ $0309	; print counter (1 byte)

block_x		equ $030a	; x-coordinate of current block
block_y		equ $030b	; y-coordinate of current block
block_dir	equ $030c	; direction of current block
block_type	equ $030d	; type of current block
block_draw_mode	equ $030e	; draw mode of current block (ff=draw, 00=clear)
block_move_mode	equ $030f	; switch between moving character and fixed character			
				; (80=moving, 00=fixed)
block_x_old	equ $0310	; old x-coordinate of block
block_y_old	equ $0311	; old y-coordinate of block
block_dir_old	equ $0312	; old direction of block
block_x_new	equ $0313	; old x-coordinate of block
block_y_new	equ $0314	; old y-coordinate of block
block_dir_new	equ $0315	; old direction of block

vram_selector	equ $0316	; switch between video RAM 1 and 2

push_left	equ $0317	; joystick is pushed left (y/n)
push_right	equ $0318	; joystick is pushed right (y/n)
push_down	equ $0319	; joystick is pushed down (y/n)
push_turn	equ $031a	; button is held down (y/n)

game_speed	equ $031b	; game speed (low value is fast)
game_speed_cnt	equ $031c	; game speed counter (counts down from game_speed to 0)
do_move_down	equ $031d	; down movement flag (y/n)

block_type_cnt	equ $031e	; pointer into array of random numbers for block types
block_type_nxt	equ $031f	; next block type

current_row	equ $0320	; current row for fullness check and deletion
current_col	equ $0321	; current column for fullness check and deletion
dfr_cnt		equ $0322	; counter for deletion of full rows

h_this_move	equ $0323	; intermediary field to save current movement
h_in_port_0	equ $0324	; interimistically holds the value of input port 0
	
credits		equ $0330	; credits (inreased by coin entry)	
score		equ $0331	; score      - 8 digits + null character (string delimiter)
hi_score	equ $033a	; high score - 8 digits + null character (string delimiter)
filled_rows	equ $0343	; counter for filled rows

auto_repeat_cnt	equ $0344	; counter for auto repeat for left/right
sound_cnt	equ $0345	; counter for music duration

rand_var	equ $0346	; random variable
rand_b		equ $0347	; intermediate result in random number generator


;**************************************************
;	**** Entry points  
;**************************************************

		org $8000	; **** ROM START

nmi_entry			; **** NMI 
		jmp nmi_start
irq_entry			; **** IRQ
		jmp irq_start
reset_entry			; **** RESET
		jmp prg_start



;**************************************************
;	**** Program  
;**************************************************

prg_start

		jsr ClearScreen
		lda #4
		jsr SetColor

		lda #DRAW		;set to draw mode
		sta block_draw_mode
		lda #MOVING		;set to moving mode
		sta block_move_mode
		lda #VRAM_2		;set to video RAM 2
		sta vram_selector
		lda #FALSE		; initialize movement flags
		sta push_left
		sta push_right
		sta push_turn
		sta push_down

		lda #FALSE
		sta do_move_down	; no down movement now

		lda #INITIAL_SPEED	; set game speed
		sta game_speed
		sta game_speed_cnt

		jsr ClearScore		; clear the scores
		jsr ClearHiScore

		cli			; enable IRQ

		jsr DrawPlayfield	; draw the playfield

		lda #0
		jsr SetColor

		ldx #17				; print the text for the preview area
		ldy #1
		lda #(str_next & $ff)	
		sta print_addr
		lda #(str_next >> 8)
		sta print_addr+1	
		jsr PrintString

		ldx #14				; print the text for the score
		ldy #SCORE_Y
		lda #(str_score & $ff)	
		sta print_addr
		lda #(str_score >> 8)
		sta print_addr+1
		jsr PrintString

		ldx #14				; print the text for the high score
		ldy #HIGH_SCORE_Y
		lda #(str_hi_score & $ff)	
		sta print_addr
		lda #(str_hi_score >> 8)
		sta print_addr+1
		jsr PrintString

		ldx #14				; print the text for the credits
		ldy #CREDITS_Y
		lda #(str_credits & $ff)	
		sta print_addr
		lda #(str_credits >> 8)
		sta print_addr+1
		jsr PrintString


		lda #7
		jsr SetColor

		ldx #14				; print the name of the game - line 1
		ldy #26
		lda #(str_vantris1 & $ff)	
		sta print_addr
		lda #(str_vantris1 >> 8)
		sta print_addr+1
		jsr PrintString

		ldx #14				; print the name of the game - line 2
		ldy #27
		lda #(str_vantris2 & $ff)	
		sta print_addr
		lda #(str_vantris2 >> 8)
		sta print_addr+1
		jsr PrintString


		lda #5
		jsr SetColor

		ldx #20				; print "by"	
		ldy #29
		lda #(str_by & $ff)	
		sta print_addr
		lda #(str_by >> 8)
		sta print_addr+1
		jsr PrintString

		ldx #14				; print my name ;-)
		ldy #31
		lda #(str_norbert & $ff)	
		sta print_addr
		lda #(str_norbert >> 8)
		sta print_addr+1
		jsr PrintString


		jmp game_over		; go to game over mode


new_game
		lda #SND_NEW_GAME	; make sound for new game
		ldx #LEN_NEW_GAME	; length of sound
		jsr MakeSound
		

		jsr ClearScore		; clear the score
	
		jsr DrawPlayfield	; draw the playfield

		jsr PrintScore		; print score

		lda #INITIAL_SPEED	; initialise game speed
		sta game_speed
		sta game_speed_cnt

		lda #0			; initialise filled rows
		sta filled_rows

		lda #FALSE
		sta do_move_down	; no down movement now

		jsr GenerateBlock	; Make the first block


main_loop
		
		jsr GetMovement		; read input device for movement

		jsr MoveBlock		; and move	

		lda do_move_down	; check if we need to move down now
		cmp #TRUE
		bne main_loop		; no, go on
	
		lda #FALSE		; yes, we move down
		sta do_move_down	; set down movement flag to false for next time

		lda #MOVE_DOWN
		jsr MoveBlock		; and move down now

		cmp #TRUE		; was down movement possible?
		beq main_loop		; yes, then go on

		lda #FIXED		; no, so block has landed 
		sta block_move_mode	; set to fixed mode,

		jsr DrawBlock		; redraw block as fixed block,

		lda #SND_LANDING	; make sound for landing
		ldx #LEN_LANDING	; length of sound
		jsr MakeSound

		lda block_y		; check if block has landed too high
		cmp #3
		bmi game_over		; game over if y < 3

		jsr DeleteFullRows	; delete rows that were filled by the landed block

		lda #1
		jsr IncScore		; each landed block is worth 10 points

		jsr PrintScore		; print score

		jsr UpdateHiScore	; update high score based on new score

		jsr PrintHiScore	; and print it

		jsr GenerateBlock	; and make new block

		jmp main_loop		; repeat main loop


game_over
		lda #3
		jsr SetColor

		ldx #1				; print empty string
		ldy #14
		lda #(str_game_over1 & $ff)	
		sta print_addr
		lda #(str_game_over1 >> 8)
		sta print_addr+1
		jsr PrintString

		ldx #1
		ldy #15
		lda #(str_game_over & $ff)	; print the string "game over"
		sta print_addr
		lda #(str_game_over >> 8)
		sta print_addr+1
		jsr PrintString

		ldx #1				; print empty string
		ldy #16
		lda #(str_game_over1 & $ff)	
		sta print_addr
		lda #(str_game_over1 >> 8)
		sta print_addr+1
		jsr PrintString

		lda #SND_GAME_OVER	; make sound for game over
		ldx #LEN_GAME_OVER	; length of sound
		jsr MakeSound


gmov_loop		
		inc rand_var		; to make it more random

		lda IN_PORT_2		; read input device

		and #$80		; start one player ?
		beq gmov_loop		; no, repeat

		lda credits		; yes, now check credits?
		cmp #0
		beq gmov_loop		; no credits, repeat

		dec credits		; yes, now credits--
		jsr PrintCredits	; show new credits

		jmp new_game		; and start the game




;**************************************************
;	**** IRQ routine - triggered by VBLANK
;**************************************************

irq_start
		php		; save the registers
		pha
		txa
		pha
		tya
		pha
		
		jsr DownMover	; start the down movement routine

		dec auto_repeat_cnt
		bne irqs_goon1

		lda #FALSE	
		sta push_left		; reset the switches for left and right to repeat
		sta push_right		; the movement if joystick is still pressed	

		lda #AUTO_REP_SPEED
		sta auto_repeat_cnt

irqs_goon1
		dec sound_cnt		; decrement sound counter
		bne irqs_goon2		; if time is over ...
		
		lda #SND_QUIET
		sta SOUND_0		; ... turn sound off

irqs_goon2

		pla			; restore registers
		tay
		pla
		tax
		pla
		plp
		
		rti


;**************************************************
;	**** NMI routine - triggered by coin insertion
;**************************************************

nmi_start
		php		; save the registers
		pha
		txa
		pha
		tya
		pha

		lda #SND_COIN		; make sound for coin insertion
		ldx #LEN_COIN		; length of sound
		jsr MakeSound
		
		inc credits	; credits++
		lda credits
		cmp #10
		bne nmi_cr_ok	; maximum credits is 9, additional coins are lost (haha :-O)

		dec credits		
nmi_cr_ok	
		jsr PrintCredits


		pla		; restore registers
		tay
		pla
		tax
		pla
		plp
		
		rti



;**************************************************
;	**** PrintCredits
;				Print current credits on the screen
;**************************************************

PrintCredits

		lda #5		; Print credits on the screen
		jsr SetColor

		clc		; convert credits to a character by adding 48
		lda credits
		adc #48		; 48 is the ASCII code for "0"
		ldx #CREDITS_X
		ldy #CREDITS_Y
		jsr PutChar	; print it on the screen

		rts


;**************************************************
;	**** ClearScore
;				Clear score
;**************************************************

ClearScore

		lda #48			; Fill all digits with "0"
		ldx #8
clsc_loop	sta score,x		; loop over digits
		dex
		bpl clsc_loop
		lda #0			; last character is null (string delimiter)
		sta score+8
		rts


;**************************************************
;	**** ClearHiScore
;				Clear high score
;**************************************************

ClearHiScore

		lda #48			; Fill all digits with "0"
		ldx #8
clhisc_loop	sta hi_score,x		; loop over digits
		dex
		bpl clhisc_loop
		lda #0			; last character is null (string delimiter)
		sta hi_score+8
		rts



;**************************************************
;	**** UpateHiScore
;				Update high score
;**************************************************

UpdateHiScore
		ldx #0 			; start with first digit
updhi_loop	lda hi_score,x		; loop over digits
		sta h_byte_A
		lda score,x
		cmp h_byte_A		; compare score with hiscore
		beq updhi_loop_goon	; if score == hi, next digit

		bpl updhi_newhi		; if score > hi, new high score
		rts			; score < hi, leave

updhi_loop_goon	inx			; score == hi, next digit
		cpx #8
		bne updhi_loop

updhi_newhi	ldx #8
updhi_loop1	lda score,x		; copy score to high score
		sta hi_score,x
		dex
		bpl updhi_loop1	

		rts



;**************************************************
;	**** IncScore		in: a (increase factor)
;				Increase score by (a times 10)
;**************************************************

IncScore
		sta h_byte_A		; save increase factor

incsc_loopf	ldx #6			; start with digit 7
incsc_loop	lda score,x		; loop over digits
		tay
		iny			; digit++
		tya
		sta score,x		; store digit
		cmp #48+10		; is it 10
		bne insc_leave		; no, then leave loop
		lda #48			; yes, then digit = 0
		sta score,x
		dex			; and increase next digit
		bpl incsc_loop		; until the highest digit is reached
insc_leave
		dec h_byte_A		; factor--
		bne incsc_loopf		; repeat until factor==0

		rts


;**************************************************
;	**** PrintScore
;				Print current score on the screen
;**************************************************

PrintScore

		lda #5			; color = 5
		jsr SetColor
		
		ldx #SCORE_X
		ldy #SCORE_Y
		lda #(score & $ff)	
		sta print_addr
		lda #(score >> 8)
		sta print_addr+1
		jsr PrintString		; print the score string

		rts


;**************************************************
;	**** PrintHiScore
	;				Print current high score on the screen
;**************************************************

PrintHiScore

		lda #5			; color = 5
		jsr SetColor
		
		ldx #HIGH_SCORE_X
		ldy #HIGH_SCORE_Y
		lda #(hi_score & $ff)	
		sta print_addr
		lda #(hi_score >> 8)
		sta print_addr+1
		jsr PrintString		; print the high score string

		rts


;**************************************************
;	**** MakeSound	in: a (sound number), x (delay time)
;			Wait a certain time and make sound in this time
;**************************************************

MakeSound

		ldy #SND_QUIET		; sound off
		sty SOUND_0

		sta SOUND_0		; sound on
		
		stx sound_cnt		; set music duration counter to length of this sound

		rts




;**************************************************
;	**** Include subroutines
;**************************************************		

	include "vantr01.asm"


;**************************************************
;	**** Include data
;**************************************************		

	include "vantrdt.asm"


;**************************************************
;	**** Pointers
;**************************************************

;	And now the interrupt and reset pointers - this ROM is mirrored to F000 by the
;	hardware, so the real address is FFFA

	org $8ffa		
	dw  nmi_entry		; NMI pointer
	dw  reset_entry		; Reset pointer
	dw  irq_entry		; IRQ pointer

	end
